@@ -1,5 +1,5 @@
 #Copyright (c) 2013 Will Harris
 # See the file license.txt for copying permission.
 
-from document import Document
+from document import Document, Draft
 
