@@ -34,6 +34,16 @@ class Link(object):
                 has no name.
     - ``title``: ``str`` containing the title of the link. Absent if the link
                  has no title.
+    - ``type``: ``str`` containing the type of the link. Absent if the link
+                does not specify a type.
+    - ``profile``: ``str`` containing the profile URL reference of the link.
+                   Absent if the link does not specify a profile.
+    - ``hreflang``: ``str`` indicating the language of the target. Absent if
+                    the link does not specify a language.
+    - ``deprecation``: ``str`` indicating that the link is deprecated. The
+                       value of the string should be a URL that provides
+                       further infomation about the deprecation. Absent if the
+                       link is not marked as deprecated.
     - ``variables``: ``list`` of names of template variables that may be
                      expanded for templated links. Empty if there are no
                      template variables.
@@ -49,6 +59,18 @@ class Link(object):
         if 'title' in json_object:
             self.title = json_object['title']
 
+        if 'type' in json_object:
+            self.type = json_object['type']
+
+        if 'profile' in json_object:
+            self.profile = json_object['profile']
+
+        if 'hreflang' in json_object:
+            self.hreflang = json_object['hreflang']
+
+        if 'deprecation' in json_object:
+            self.deprecation = json_object['deprecation']
+
         self.is_templated = self.o.get('templated', False) == True
 
         if self.is_templated:
