@@ -124,7 +124,7 @@ class ParseEmbeddedObjectsTest(unittest.TestCase):
 class CurieExpansionTest(unittest.TestCase):
     OBJECT = {
         '_links': {
-            'curie': [
+            'curies': [
                 {
                     'href': "http://localhost/roles/{rel}",
                     'name': 'role',
@@ -141,10 +141,9 @@ class CurieExpansionTest(unittest.TestCase):
         '_embedded': {
             'role:sizing': {
                 '_links': {
-                    'curie': [
+                    'curies': [
                         {
-                            'href':
-                            "http://localhost/dimension/{rel}",
+                            'href': "http://localhost/dimension/{rel}",
                             'name': 'dim',
                             'templated': True
                         }
@@ -153,7 +152,7 @@ class CurieExpansionTest(unittest.TestCase):
             },
             'role:coloring': {
                 '_links': {
-                    'curie': [
+                    'curies': [
                         {
                             'href':
                             "http://localhost/imagefiles/{rel}",
@@ -191,11 +190,50 @@ class CurieExpansionTest(unittest.TestCase):
         self.assertEquals("http://localhost/imagefiles/photo",
                           coloring_doc.expand_curie('image:photo'))
 
+class CurieExpansionTestDraft3(CurieExpansionTest):
+    OBJECT = {
+        '_links': {
+            'curie': [
+                {
+                    'href': "http://localhost/roles/{rel}",
+                    'name': 'role',
+                    'templated': True
+                },
+                {
+                    'href': "http://localhost/images/{rel}",
+                    'name': 'image',
+                    'templated': True
+                }
+            ],
+            'role:host': {'href': "/hosts/1"}
+        },
+        '_embedded': {
+            'role:sizing': {
+                '_links': {
+                    'curie': {
+                        'href': "http://localhost/dimension/{rel}",
+                        'name': 'dim',
+                        'templated': True
+                    }
+                }
+            },
+            'role:coloring': {
+                '_links': {
+                    'curie': {
+                        'href': "http://localhost/imagefiles/{rel}",
+                        'name': 'image',
+                        'templated': True
+                    }
+                }
+            },
+        }
+    }
+
 
 class RelsTest(unittest.TestCase):
     OBJECT = {
         '_links': {
-            'curie': [
+            'curies': [
                 {
                     'href': "/roles/{rel}",
                     'name': 'role',
@@ -286,6 +324,60 @@ class RelsTest(unittest.TestCase):
             urls)
 
 
+class RelsTestDraft3(RelsTest):
+    OBJECT = {
+        '_links': {
+            'curie': [
+                {
+                    'href': "/roles/{rel}",
+                    'name': 'role',
+                    'templated': True
+                },
+                {
+                    'href': "http://localhost/images/{rel}",
+                    'name': 'image',
+                    'templated': True
+                }
+            ],
+            'role:host': {'href': "/hosts/1"},
+            'role:application': {'href': "/apps/1"},
+            'role:dept': [
+                {'href': "/departments/1"},
+                {'href': "/departments/2"}
+            ]
+        },
+        '_embedded': {
+            'role:consumer': {
+                '_links': {
+                    'self': {
+                        'href': '/clients/1'
+                    }
+                },
+                'name': "Client 1"
+            },
+            'role:application': {
+                '_links': {
+                    'self': { 'href': "/apps/2" }
+                }
+            },
+            'role:dept': [
+                {
+                    '_links': {
+                        'self': {
+                            'href': "http://localhost/departments/2"
+                        }
+                    },
+                },
+                {
+                    '_links': {
+                        'self': { 'href': "/departments/3" }
+                    }
+                }
+            ]
+        }
+    }
+
+
 class SerializeTests(unittest.TestCase):
     def checkEqualObjects(self, obj):
         doc = dougrain.Document.from_object(obj, "http://localhost")
@@ -310,6 +402,10 @@ class SerializeTests(unittest.TestCase):
         self.checkEqualObjects(CurieExpansionTest.OBJECT)
         self.checkEqualObjects(RelsTest.OBJECT)
 
+    def testRelsDraft3(self):
+        self.checkEqualObjects(CurieExpansionTestDraft3.OBJECT)
+        self.checkEqualObjects(RelsTestDraft3.OBJECT)
+
 
 class AttributeMutationTests(unittest.TestCase):
     def testSetAttributeAddsAttribute(self):
@@ -696,23 +792,24 @@ class TestIteration(unittest.TestCase):
         self.assertEquals(1, count)
 
 
-def make_doc(href):
-    result = dougrain.Document.empty("http://localhost")
+def make_doc(href, *args, **kwargs):
+    result = dougrain.Document.empty("http://localhost", *args, **kwargs)
     result.add_link('self', href)
     return result
 
 
+def make_draft_3_doc(href, *args, **kwargs):
+    return make_doc(draft=dougrain.Draft.DRAFT_3)
+
+
 class DeleteEmbeddedTests(unittest.TestCase):
-    def doc(self, href):
-        return make_doc(href)
-
     def testDeleteOnlyEmbedForRel(self):
-        doc = self.doc("http://localhost/2")
-        doc.embed('child', self.doc("http://localhost/2/1"))
-        doc.embed('root', self.doc("http://localhost/"))
+        doc = make_doc("http://localhost/2")
+        doc.embed('child', make_doc("http://localhost/2/1"))
+        doc.embed('root', make_doc("http://localhost/"))
 
-        target_doc = self.doc("http://localhost/2")
-        target_doc.embed('root', self.doc("http://localhost/"))
+        target_doc = make_doc("http://localhost/2")
+        target_doc.embed('root', make_doc("http://localhost/"))
 
         doc.delete_embedded("child")
 
@@ -721,14 +818,14 @@ class DeleteEmbeddedTests(unittest.TestCase):
         self.assertTrue('root' in doc.embedded)
 
     def testDeleteEveryEmbedForRel(self):
-        doc = self.doc("http://localhost/2")
-        doc.embed('root', self.doc("http://localhost/"))
-        doc.embed('child', self.doc("http://localhost/2/1"))
-        doc.embed('child', self.doc("http://localhost/2/1"))
-        doc.embed('child', self.doc("http://localhost/2/1"))
+        doc = make_doc("http://localhost/2")
+        doc.embed('root', make_doc("http://localhost/"))
+        doc.embed('child', make_doc("http://localhost/2/1"))
+        doc.embed('child', make_doc("http://localhost/2/1"))
+        doc.embed('child', make_doc("http://localhost/2/1"))
 
-        target_doc = self.doc("http://localhost/2")
-        target_doc.embed('root', self.doc("http://localhost/"))
+        target_doc = make_doc("http://localhost/2")
+        target_doc.embed('root', make_doc("http://localhost/"))
 
         doc.delete_embedded("child")
 
@@ -737,11 +834,11 @@ class DeleteEmbeddedTests(unittest.TestCase):
         self.assertTrue('root' in doc.embedded)
 
     def testDeleteLastEmbed(self):
-        doc = self.doc("http://localhost/2")
-        doc.embed('root', self.doc("http://localhost/"))
-        doc.embed('child', self.doc("http://localhost/2/1"))
+        doc = make_doc("http://localhost/2")
+        doc.embed('root', make_doc("http://localhost/"))
+        doc.embed('child', make_doc("http://localhost/2/1"))
 
-        target_doc = self.doc("http://localhost/2")
+        target_doc = make_doc("http://localhost/2")
 
         doc.delete_embedded('root')
         doc.delete_embedded('child')
@@ -749,25 +846,25 @@ class DeleteEmbeddedTests(unittest.TestCase):
         self.assertEquals(target_doc.as_object(), doc.as_object())
 
     def testDeleteIndividualEmbeds(self):
-        doc = self.doc("http://localhost/2")
-        doc.embed('root', self.doc("http://localhost/"))
-        doc.embed('child', self.doc("http://localhost/2/1"))
-        doc.embed('child', self.doc("http://localhost/2/2"))
-        doc.embed('child', self.doc("http://localhost/2/3"))
+        doc = make_doc("http://localhost/2")
+        doc.embed('root', make_doc("http://localhost/"))
+        doc.embed('child', make_doc("http://localhost/2/1"))
+        doc.embed('child', make_doc("http://localhost/2/2"))
+        doc.embed('child', make_doc("http://localhost/2/3"))
 
-        doc2 = self.doc("http://localhost/2")
-        doc2.embed('root', self.doc("http://localhost/"))
-        doc2.embed('child', self.doc("http://localhost/2/2"))
-        doc2.embed('child', self.doc("http://localhost/2/3"))
+        doc2 = make_doc("http://localhost/2")
+        doc2.embed('root', make_doc("http://localhost/"))
+        doc2.embed('child', make_doc("http://localhost/2/2"))
+        doc2.embed('child', make_doc("http://localhost/2/3"))
 
-        doc3 = self.doc("http://localhost/2")
-        doc3.embed('root', self.doc("http://localhost/"))
-        doc3.embed('child', self.doc("http://localhost/2/2"))
+        doc3 = make_doc("http://localhost/2")
+        doc3.embed('root', make_doc("http://localhost/"))
+        doc3.embed('child', make_doc("http://localhost/2/2"))
 
-        doc4 = self.doc("http://localhost/2")
-        doc4.embed('root', self.doc("http://localhost/"))
+        doc4 = make_doc("http://localhost/2")
+        doc4.embed('root', make_doc("http://localhost/"))
 
-        doc5 = self.doc("http://localhost/2")
+        doc5 = make_doc("http://localhost/2")
 
         doc.delete_embedded("child", "http://localhost/2/1")
         self.assertEquals(doc2.as_object(), doc.as_object())
@@ -782,40 +879,40 @@ class DeleteEmbeddedTests(unittest.TestCase):
         self.assertEquals(doc5.as_object(), doc.as_object())
 
     def testDeleteEmbedsWithoutRel(self):
-        doc = self.doc("http://localhost/3")
-        doc.embed('child', self.doc("http://localhost/3/1"))
-        doc.embed('child', self.doc("http://localhost/3/2"))
-        doc.embed('favorite', self.doc("http://localhost/3/1"))
+        doc = make_doc("http://localhost/3")
+        doc.embed('child', make_doc("http://localhost/3/1"))
+        doc.embed('child', make_doc("http://localhost/3/2"))
+        doc.embed('favorite', make_doc("http://localhost/3/1"))
 
-        target_doc = self.doc("http://localhost/3")
-        target_doc.embed('child', self.doc("http://localhost/3/2"))
+        target_doc = make_doc("http://localhost/3")
+        target_doc.embed('child', make_doc("http://localhost/3/2"))
 
         doc.delete_embedded(href="http://localhost/3/1")
 
         self.assertEquals(target_doc.as_object(), doc.as_object())
 
     def testDeleteAllEmbeds(self):
-        doc = self.doc("http://localhost/3")
-        doc.embed('child', self.doc("http://localhost/3/1"))
-        doc.embed('child', self.doc("http://localhost/3/2"))
-        doc.embed('favorite', self.doc("http://localhost/3/1"))
+        doc = make_doc("http://localhost/3")
+        doc.embed('child', make_doc("http://localhost/3/1"))
+        doc.embed('child', make_doc("http://localhost/3/2"))
+        doc.embed('favorite', make_doc("http://localhost/3/1"))
 
-        target_doc = self.doc("http://localhost/3")
+        target_doc = make_doc("http://localhost/3")
 
         doc.delete_embedded()
 
         self.assertEquals(target_doc.as_object(), doc.as_object())
 
     def testDeleteEmbedWithMissingRel(self):
-        doc = self.doc("http://localhost/3")
-        doc.embed('child', self.doc("http://localhost/3/1"))
-        doc.embed('child', self.doc("http://localhost/3/2"))
-        doc.embed('favorite', self.doc("http://localhost/3/1"))
+        doc = make_doc("http://localhost/3")
+        doc.embed('child', make_doc("http://localhost/3/1"))
+        doc.embed('child', make_doc("http://localhost/3/2"))
+        doc.embed('favorite', make_doc("http://localhost/3/1"))
 
-        target_doc = self.doc("http://localhost/3")
-        target_doc.embed('child', self.doc("http://localhost/3/1"))
-        target_doc.embed('child', self.doc("http://localhost/3/2"))
-        target_doc.embed('favorite', self.doc("http://localhost/3/1"))
+        target_doc = make_doc("http://localhost/3")
+        target_doc.embed('child', make_doc("http://localhost/3/1"))
+        target_doc.embed('child', make_doc("http://localhost/3/2"))
+        target_doc.embed('favorite', make_doc("http://localhost/3/1"))
 
         missing_rel = ''.join(doc.embedded.keys()) + '_'
         doc.delete_embedded(missing_rel)
@@ -823,8 +920,8 @@ class DeleteEmbeddedTests(unittest.TestCase):
         self.assertEquals(target_doc.as_object(), doc.as_object())
 
     def testDeleteEmbedWithNoEmbeds(self):
-        doc = self.doc("http://localhost/3")
-        target_doc = self.doc("http://localhost/3")
+        doc = make_doc("http://localhost/3")
+        target_doc = make_doc("http://localhost/3")
 
         doc.delete_embedded()
 
@@ -832,37 +929,68 @@ class DeleteEmbeddedTests(unittest.TestCase):
 
 
 class CurieMutationTest(unittest.TestCase):
-    def testSetCurie(self):
-        doc = make_doc("http://localhost/3")
-        doc.set_curie('rel', "http://localhost/rels/{rel}")
+    def setUp(self):
+        self.doc = make_doc("http://localhost/3")
+        self.doc.set_curie('rel', "http://localhost/rels/{rel}")
 
-        new_doc = dougrain.Document(doc.as_object(), doc.base_uri)
+    def testCurieJSONHasCorrectType(self):
+        self.assertEquals(type(self.doc.as_object()['_links']['curies']), list)
+
+    def testSetCurie(self):
+        new_doc = dougrain.Document(self.doc.as_object(), self.doc.base_uri)
         self.assertEquals("http://localhost/rels/foo",
                           new_doc.expand_curie("rel:foo"))
 
     def testReplaceCurie(self):
-        doc = make_doc("http://localhost/3")
-        doc.set_curie('rel', "http://localhost/rels/{rel}")
-        doc.set_curie('rel', "http://localhost/RELS/{rel}.html")
+        self.doc.set_curie('rel', "http://localhost/RELS/{rel}.html")
 
-        new_doc = dougrain.Document(doc.as_object(), doc.base_uri)
+        new_doc = dougrain.Document(self.doc.as_object(), self.doc.base_uri)
         self.assertEquals("http://localhost/RELS/foo.html",
                           new_doc.expand_curie("rel:foo"))
 
     def testDropCurie(self):
-        doc = make_doc("http://localhost/3")
-        doc.set_curie('rel', "http://localhost/rels/{rel}")
-        doc.set_curie('tm', "http://www.touchmachine.com/{rel}.html")
-        doc.drop_curie('rel')
+        self.doc.set_curie('tm', "http://www.touchmachine.com/{rel}.html")
+        self.doc.drop_curie('rel')
 
-        new_doc = dougrain.Document(doc.as_object(), doc.base_uri)
-        self.assertEquals("rel:foo", doc.expand_curie("rel:foo"))
+        new_doc = dougrain.Document(self.doc.as_object(), self.doc.base_uri)
+        self.assertEquals("rel:foo", self.doc.expand_curie("rel:foo"))
         self.assertEquals("http://www.touchmachine.com/index.html",
                           new_doc.expand_curie("tm:index"))
 
+    def testDropSingleCurie(self):
+        self.doc.drop_curie('rel')
+        self.assertEquals("rel:foo", self.doc.expand_curie("rel:foo"))
+
+
+class CurieMutationTestDraft3(CurieMutationTest):
+    def setUp(self):
+        self.doc = dougrain.Document.empty("http://localhost/3",
+                                           draft=dougrain.Draft.DRAFT_3)
+        self.doc.set_curie('rel', "http://localhost/rels/{rel}")
+        self.doc.add_link('self', "http://localhost/3")
+
+    def testCurieJSONHasCorrectType(self):
+        self.assertEquals(type(self.doc.as_object()['_links']['curie']), dict)
+
 
 class CurieHidingTests(unittest.TestCase):
     def testCuriesAreNotLinks(self):
+        doc = dougrain.Document({
+            '_links': {
+                'curies': [{
+                    'href': "http://localhost/rel/{rel}",
+                    'name': "rel",
+                    'templated': True
+                }],
+                'self': {
+                    'href': "http://localhost/0"
+                }
+            }
+        }, "http://localhost/0")
+
+        self.assertFalse('curies' in doc.links)
+
+    def testDraft3CuriesAreNotLinks(self):
         doc = dougrain.Document({
             '_links': {
                 'curie': {
@@ -877,6 +1005,28 @@ class CurieHidingTests(unittest.TestCase):
         }, "http://localhost/0")
 
         self.assertFalse('curie' in doc.links)
+
+    def testDraft3CuriesAreLinksInDraft4Document(self):
+        doc = dougrain.Document({
+            '_links': {
+                'curie': {
+                    'href': "http://localhost/rel/{rel}",
+                    'name': "rel",
+                    'templated': True
+                },
+                'curies': [{
+                    'href': "http://localhost/rel/{rel}",
+                    'name': "rel",
+                    'templated': True
+                }],
+                'self': {
+                    'href': "http://localhost/0"
+                }
+            }
+        }, "http://localhost/0")
+
+        self.assertFalse('curies' in doc.links)
+        self.assertTrue('curie' in doc.links)
                 
 
 class LinkCanonicalizationTests(unittest.TestCase):
@@ -906,7 +1056,7 @@ class LinkCanonicalizationTests(unittest.TestCase):
         self.doc = dougrain.Document.from_object(
             {
                 "_links": {
-                    "curie": [
+                    "curies": [
                         {"href": "/roles/{rel}",
                          "name": "role",
                          "templated": True}
@@ -923,6 +1073,27 @@ class LinkCanonicalizationTests(unittest.TestCase):
         self.assertEquals(set(["/apps/1", "/apps/2", "/apps/3"]),
                           set(link.href for link in self.doc.links["role:app"]))
 
+    def testMergesLinkswhenLoadingDraft3(self):
+        self.doc = dougrain.Document.from_object(
+            {
+                "_links": {
+                    "curie": {
+                        "href": "/roles/{rel}",
+                        "name": "role",
+                        "templated": True
+                    },
+                    "self": {"href": "/1"},
+                    "role:app": {"href": "/apps/1"},
+                    "/roles/app": {"href": "/apps/2"},
+                    "http://localhost/roles/app": {"href": "/apps/3"},
+                }
+            },
+            base_uri="http://localhost/1")
+        self.doc.set_curie("role", "/roles/{rel}")
+        
+        self.assertEquals(set(["/apps/1", "/apps/2", "/apps/3"]),
+                          set(link.href for link in self.doc.links["role:app"]))
+
 
 class EmbeddedCanonicalizationTests(unittest.TestCase):
     def setUp(self):
@@ -991,5 +1162,79 @@ class EdgeCasesTests(unittest.TestCase):
         self.assertEquals("bar", doc.embedded['child'].properties['foo'])
 
 
+class ExplicitDraftTests(unittest.TestCase):
+    def testDraft3DocumentHasOldCurieBehaviour(self):
+        doc = dougrain.Document.empty("http://localhost",
+                                      draft=dougrain.Draft.DRAFT_3)
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_3)
+        doc.add_link('self', "/1")
+        doc.set_curie('rel', "/rels/{rel}")
+
+        links = doc.as_object()['_links']
+        self.assertTrue('curie' in links)
+        self.assertFalse('curies' in links)
+        curie = links['curie']
+        self.assertTrue(isinstance(curie, dict))
+        doc.set_curie('foo', "/foos/{rel}")
+        curie = links['curie']
+        self.assertTrue(isinstance(curie, list))
+
+    def testDraft4DocumentHasNewCurieBehaviour(self):
+        doc = dougrain.Document.empty("http://localhost",
+                                      draft=dougrain.Draft.DRAFT_4)
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_4)
+        doc.add_link('self', "/1")
+        doc.set_curie('rel', "/rels/{rel}")
+
+        links = doc.as_object()['_links']
+        self.assertFalse('curie' in links)
+        self.assertTrue('curies' in links)
+        curie = links['curies']
+        self.assertTrue(isinstance(curie, list))
+        doc.set_curie('foo', "/foos/{rel}")
+        curie = links['curies']
+        self.assertTrue(isinstance(curie, list))
+
+
+class DraftDetectionTests(unittest.TestCase):
+    def testDocumentsWithCurieAreDraft3(self):
+        doc = dougrain.Document.from_object(
+            {'_links': {'curie': {}}},
+            "http://localhost/")
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_3)
+
+    def testDocumentsWithCuriesAreDraft4(self):
+        doc = dougrain.Document.from_object(
+            {'_links': {'curies': []}},
+            "http://localhost/")
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_4)
+
+    def testDocumentsWithNoCurieKeyAreLatest(self):
+        doc = dougrain.Document.empty("http://localhost/")
+        self.assertEquals(doc.draft, dougrain.Draft.LATEST)
+
+    def testLatestDraftIsDraft4(self):
+        doc = dougrain.Document.from_object({}, "http://localhost/",
+                                            draft=dougrain.Draft.LATEST)
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_4)
+
+    def testExplicitDraftOverridesAutodetection(self):
+        doc = dougrain.Document.from_object(
+            {'_links': {'curie': {"href": "/roles/{rel}",
+                                  "name": "role",
+                                  "templated": True}}},
+            "http://localhost/",
+            draft=dougrain.Draft.DRAFT_4
+        )
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_4)
+
+        doc = dougrain.Document.from_object(
+            {'_links': {'curies': []}},
+            "http://localhost/",
+            draft=dougrain.Draft.DRAFT_3
+        )
+        self.assertEquals(doc.draft, dougrain.Draft.DRAFT_3)
+
+
 if __name__ == '__main__':
     unittest.main()
